--!parent

--------------------------------------
-- Opens file with specified mode, returns file pointer (xtra instance)
-- @param {string} tFile - path to file
-- @param {string} tMode - r, rb, w, wb, a, ab, rw, rwb, wr, wrb
-- @return {instance|false}
--------------------------------------
on fopen (me, tFile, tMode)
	fp = xtra("BinFile").new()
	fp.bx_fopen(tFile, tMode)
	return fp
end

--------------------------------------
-- Closes open file
-- @param {instance} fp
--------------------------------------
on fclose (me, fp)
	fp.bx_fclose()
end

----------------------------------------
-- Deletes open file
-- @param {instance} fp
-- @return {bool}
----------------------------------------
on fdelete (me, fp)
	return fp.bx_fdelete()
end

----------------------------------------
-- Reads n bytes from open file, returns bytearray
-- @param {instance} fp
-- @param {integer} n
-- @return {bytearray}
----------------------------------------
on freadbytes(me, fp, n)
	return byteArray(fp.bx_fread(n))
end

----------------------------------------
-- Writes byteArray to open file
-- @param {instance} fp
-- @param {bytearray} ba
----------------------------------------
on fwritebytes (me, fp, ba)
	return fp.bx_fwrite(ba._data)
end

----------------------------------------
-- Reads n bytes from open file, returns string
-- @param {instance} fp
-- @param {integer} n
-- @return {string}
----------------------------------------
on freadstring(me, fp, n)
	return fp.bx_fread(n)
end

----------------------------------------
-- Writes string to open file
-- @param {instance} fp
-- @param {string} str
----------------------------------------
on fwritestring (me, fp, str)
	fp.bx_fwrite(str)
end

--------------------------------------
-- Returns position in open file
-- @param {instance} fp
-- @return {integer}
--------------------------------------
on ftell (me, fp)
	return fp.bx_ftell()
end

--------------------------------------
-- Sets position in open file
-- whence: either integer or symbol
-- 0 or #SEEK_SET: set position to offset bytes. (default)
-- 1 or #SEEK_CUR: set position to current position plus offset.
-- 2 or #SEEK_END: set position to end of file plus offset.
-- @param {instance} fp
-- @param {integer} pos
-- @param {integer|symbol} [whence=0]
-- @return {bool}
--------------------------------------
on fseek (me, fp, pos, whence)
	if voidP(whence) then whence=0
	return fp.bx_fseek(pos, whence)
end

--------------------------------------
-- Returns size of open file
-- @param {instance} fp
-- @return {integer}
--------------------------------------
on fsize (me, fp)
	return fp.bx_fsize()
end

----------------------------------------
-- Reads 32-bit integer from open file
-- @param {instance} fp
-- @param {symbol} endian - optional, default = #LittleEndian
-- @return {integer}
----------------------------------------
on freadint32 (me, fp, endian)
	ba = me.freadbytes(fp, 4)
	if not voidP(endian) then ba.endian = endian
	return ba.readInt32()
end

----------------------------------------
-- Writes 32-bit integer to open file
-- @param {instance} fp
-- @param {integer} n
-- @param {symbol} endian - optional, default = #LittleEndian
----------------------------------------
on fwriteint32 (me, fp, n, endian)
	ba = bytearray(4)
	if not voidP(endian) then ba.endian = endian
	ba.writeInt32(n)
	me.fwritebytes(fp, ba)
end

----------------------------------------
-- Reads 16-bit integer from open file
-- @param {instance} fp
-- @return {integer}
----------------------------------------
on freadint16 (me, fp)
	ba = me.freadbytes(fp, 2)
	return ba.readInt16()
end

----------------------------------------
-- Writes 16-bit integer to open file
-- @param {instance} fp
-- @param {integer} n
----------------------------------------
on fwriteint16 (me, fp, n)
	ba = bytearray(2)
	ba.writeInt16(n)
	me.fwritebytes(fp, ba)
end

----------------------------------------
-- Reads 8-bit integer from open file
-- @param {instance} fp
-- @return {integer}
----------------------------------------
on freadint8 (me, fp)
	return chartonum(fp.bx_fread(1))
end

----------------------------------------
-- Writes 8-bit integer to open file
-- @param {instance} fp
-- @param {integer} n
----------------------------------------
on fwriteint8 (me, fp, n)
	fp.bx_fwrite(numtochar(n))
end

----------------------------------------
-- Copies data from fp_src to fp_dest, using a memory friendly buffer
-- @param {instance} fp_src
-- @param {instance} fp_dest
-- @param {integer} size
-- @param {integer} buffsize - optional, default = 10 KB buffer
----------------------------------------
on fcopybytes (me, fp_src, fp_dest, size, buffsize)
	if voidP(buffsize) then buffsize = 10*1024 -- 10 KB
	cnt = size/buffsize
	repeat with i = 1 to cnt
		fp_dest.bx_fwrite(fp_src.bx_fread(buffsize))
	end repeat
	if size>cnt*buffsize then
		fp_dest.bx_fwrite(fp_src.bx_fread(size-cnt*buffsize))
	end if
end

--**************************************
-- global functions, no file pointer passed
--**************************************

----------------------------------------
-- Returns file size
-- @param {string} tFile
-- @return {integer}
----------------------------------------
on size (me, tFile)
	return bx_file_size(tFile)
end

----------------------------------------
-- Truncates file
-- @param {string} tFile
-- @param {integer} tSize
-- @return {bool}
----------------------------------------
on truncate (me, tFile, tSize)
	return bx_file_truncate(tFile, tSize)
end

----------------------------------------
-- Deletes file
-- @param {string} tFile
-- @return {bool}
----------------------------------------
on delete (me, tFile)
	return bx_file_delete(tFile)
end

----------------------------------------
-- Recycle file (not availble in original fileIO lib!)
-- @param {string} tFile
-- @return {bool}
----------------------------------------
--on recycle (me, tFile)
--	return bx_file_recycle(tFile)
--end

----------------------------------------
-- is_file() ?
-- @param {string} tFile
-- @return {bool}
----------------------------------------
on exists (me, tFile)
	return bx_exists(tFile)
end

----------------------------------------
-- Reads whole file, returns string
-- @param {string} tFile
-- @return {string|false}
----------------------------------------
on getString (me, tFile, tEncoding)
--	if voidP(tEncoding) then tEncoding="utf-8"
--	ba = bx_file_get_contents(tFile)
--	return ba.readRawString(ba.length, tEncoding)
	return bx_file_get_contents(tFile)
end

----------------------------------------
-- Saves string as file
-- @param {string} tFile
-- @param {string} tString
-- @return {bool}
----------------------------------------
on putString (me, tFile, tString)
	return bx_file_put_contents(tFile, tString)
end

----------------------------------------
-- Returns file as ByteArray
-- @param {string} tFile
-- @return {byteArray|false}
----------------------------------------
on getBytes (me, tFile)
	return byteArray(bx_file_get_contents(tFile))
end

----------------------------------------
-- Saves ByteArray to file
-- @param {string} tFile
-- @param {byteArray} tByteArray
-- @param {integer} [tLength=VOID]
-- @return {bool}
----------------------------------------
on putBytes (me, tFile, tByteArray, tLength)
	if integerP(tLength) then
		--return bx_file_put_contents(tFile, tByteArray._data.char[tByteArray.position..tByteArray.position + tLength - 1])
		return bx_file_put_contents(tFile, tByteArray._data.char[1..tLength - 1])
	else
		return bx_file_put_contents(tFile, tByteArray._data)
	end if
end

----------------------------------------
-- Displays open file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
--on selectFileOpen (me, tTitle, filterMask, defaultFileName)
--	return string(bx_display_open(tTitle, filterMask, defaultFileName))
--end

----------------------------------------
-- Displays save file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
--on selectFileSave (me, tTitle, filterMask, defaultFileName)
--	return string(bx_display_save(tTitle, filterMask, defaultFileName))
--end

----------------------------------------
-- Select folder via save file dialog
-- @param {string} tTitle
-- @return {string}
----------------------------------------
--on selectFolder (me, tTitle)
--	if voidP(tTitle) then tTitle = "Select Folder"
--	fn = bx_display_save(tTitle, "*.*")
--	if string(fn)=EMPTY then return ""
--	pd = the last char of _movie.path
--	od = _player.itemDelimiter
--	_player.itemDelimiter = pd
--	delete the last item of fn
--	_player.itemDelimiter = od
--	put pd after fn
--	return fn
--end
