//!parent
//!encoding=utf-8

//****************************************************************************
// @file LSW CORE LIB: REGEXP
// @author Valentin Schmidt
// @version 0.10
//****************************************************************************

/**
 * Returns a RegEx object.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} [mod='g'] - Modifier string
 * @return {object}
 */
function re (me, patt, mod){
	if (mod==null) mod='g';
	return new RegExp(patt, mod);
}

/**
 * Escapes string for regexp usage
 * @param {string}
 * @return {number}
 */
function escape (me, str){
	return str.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}

/**
 * Executes a search for a match in a specified string, returns list or VOID.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {number} The position of first occurence (or -1)
 */
function exec (me, patt, str, mod){
	if (mod==null) mod='g';
	var re = new RegExp(patt, mod);
	var arr = re.exec(str);
	if (arr){
		var tList = list();
		for (var i in arr) tList.add(arr[i]);
		return tList;
	}
}

/**
 * Tests for match of pattern in string, returns true or false
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {bool}
 */
function test (me, patt, str, mod){
	if (mod==null) mod='g';
	var re = new RegExp(patt, mod);
	return re.test(str);
}

/**
 * Searches for first occurence of pattern in string, returns position
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {number} The position of first occurence (or -1)
 */
function search_pos (me, patt, str, mod){
	if (mod==null) mod='g';
	return str.search(new RegExp(patt, mod));
}

/**
 * Splits string according to pattern, returns list
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Lingo list
 */
function split (me, patt, str, mod){
	if (mod==null) mod='g';
	var arr = str.split(new RegExp(patt, mod));
	var tList = list();
	for (var i in arr) tList.add(arr[i]);
	return tList;
}

/**
 * Retrieves the matches when matching a string against a regular expression.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Lingo list
 */
function match (me, patt, str, mod){
	if (mod==null) mod='g';
	var arr = str.match(new RegExp(patt, mod));
	var tList = list();
	for (var i in arr) tList.add(arr[i]);
	return tList;
}

function re_match (me, re, str){
	var arr = str.match(re);
	var tList = list();
	for (var i in arr) tList.add(arr[i]);
	return tList;
}

/**
 * Retrieves matches for specfic capture when matching a string against a regular expression.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @param {integer} [index=0] - Capture index
 * @return {list} Lingo list
 */
function match_index (me, patt, str, mod, index){
	if (mod==null) mod='g';
	if (index==null) index=0;
	var re = new RegExp(patt, mod)
	var arr;
	var tList = list();
	while ((arr = re.exec(str)) !== null) {
		tList.add(arr[index]);
	}
	return tList;
}

function re_match_index (me, re, str, index){
	if (index==null) index=0;
	var arr;
	var tList = list();
	while ((arr = re.exec(str)) !== null) {
		tList.add(arr[index]);
	}
	return tList;
}

/**
 * Retrieves matches for specified captures when matching a string against a regular expression.
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @param {list} indexes - Capture indexes
 * @return {list} Lingo list (of lists)
 */
function match_indexes (me, patt, str, mod, indexes){
	if (mod==null) mod='g';
	var re = new RegExp(patt, mod)
	var arr;
	var tList = list();
	var cnt = indexes.count;
	for (var i=1;i<=cnt;i++) tList[i] = list();
	while ((arr=re.exec(str))!==null) {
		for (var i=1;i<=cnt;i++) tList[i].add(arr[indexes[i]]);
	}
	return tList;
}

/**
 * Replaces pattern matches with string
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} replStr - The replace string
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace (me, patt, replStr, str, mod){
	if (mod==null) mod='g';
	return str.replace(new RegExp(patt, mod), replStr);
}

function re_replace (me, re, replStr, str){
	return str.replace(re, replStr);
}

/**
 * Replaces pattern matches with strings
 * Not binary-safe!
 * @param {string} pattList - List of RegEx patterns (as strings)
 * @param {string} replStrList - List of replace strings
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace_multi (me, pattList, replStrList, str, mod){
	if (mod==null) mod='g';
	var i, len = pattList.count;
	for (i=1;i<=len;i++){
		str = str.replace(new RegExp(pattList[i], mod), replStrList[i]);
	}
	return str;
}

function re_replace_multi (me, reList, replStrList, str){
	var i, len = reList.count;
	for (i=1;i<=len;i++){
		str = str.replace(reList[i], replStrList[i]);
	}
	return str;
}

/**
 * Replaces pattern matches with strings specified by hash (JS object or Lingo propList)
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {object|propList} hash - The replacement hash
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @return {string} The result string
 */
function replace_hash (me, patt, hash, str, mod){
	if (mod==null) mod='g';
	return str.replace(new RegExp(patt, mod), function(c){return hash[c]});
}

function re_replace_hash (me, re, hash, str){
	return str.replace(re, function(c){return hash[c]});
}

/**
 * Replaces pattern matches with strings specified by hash (JS object or Lingo propList)
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {object|propList} hash - The replacement hash
 * @param {string} str - The input string
 * @param {string} [mod='g'] - Modifier string
 * @param {integer} [index=0] - Capture index
 * @return {string} The result string
 */
function replace_hash_index (me, patt, hash, str, mod, index){
	if (mod==null) mod='g';
	if (index==null) index=0;
	return str.replace(new RegExp(patt, mod), function(c){return hash[arguments[index]]});
}

function re_replace_hash_index (me, re, hash, str, index){
	if (index==null) index=0;
	return str.replace(re, function(c){return hash[arguments[index]]});
}

/**
 * Replaces pattern matches with strings returned by specified callback
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {symbol} cbHandler - The callback function
 * @param {object} [cbObject=_movie]
 * @param {string} [mod='g'] - Modifier string
 * @param {any} [data] - custom data passed to callback
 * @return {string} The result string
 */
function replace_callback (me, patt, str, cbHandler, cbObject, mod, data){
	if (!cbObject) cbObject=_movie;
	if (mod==null) mod='g';
	return str.replace(new RegExp(patt, mod), function(){
		var args = list();
		for (var i=0;i<arguments.length;i++) args.append(arguments[i]);
		return _movie.call(cbHandler, list(cbObject), args, data);
	});
}

function re_replace_callback (me, re, str, cbHandler, cbObject, data){
	if (!cbObject) cbObject=_movie;
	return str.replace(re, function(){
		var args = list();
		for (var i=0;i<arguments.length;i++) args.append(arguments[i]);
		return _movie.call(cbHandler, list(cbObject), args, data);
	});
}

/**
 * Replaces pattern matches of specific index with strings returned by specified callback
 * Not binary-safe!
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {integer} index - Capture index
 * @param {symbol} cbHandler - The callback function
 * @param {object} [cbObject=_movie]
 * @param {string} [mod='g'] - Modifier string
 * @param {any} [data] - custom data passed to callback
 * @return {string} The result string
 */
function replace_callback_index (me, patt, str, index, cbHandler, cbObject, mod, data){
	if (!cbObject) cbObject=_movie;
	if (mod==null) mod='g';
	return str.replace(new RegExp(patt, mod), function(){
		return _movie.call(cbHandler, cbObject, arguments[index], data);
	});
}

function re_replace_callback_index (me, re, str, index, cbHandler, cbObject, data){
	if (!cbObject) cbObject=_movie;
	return str.replace(re, function(){
		return _movie.call(cbHandler, cbObject, arguments[index], data);
	});
}

/**
 * Searches for pattern in string
 * @param {string} patt - The RegEx pattern (as string)
 * @param {string} str - The input string
 * @param {integer} [ref] - Ref of capture group that should be returned (default = 0)
 * @param {list|null} [indexes] - If a list is passed, it's filled with indexes of search results
 * @param {string} [mod='g'] - Modifier string
 * @return {list} Found matches
 */
function search (me, patt, str, ref, indexes, mod){
	if (ref==null) ref=0;
	if (mod==null) mod='g';
	var re = new RegExp(patt, mod);
	var matches = list();
	if (typeof indexes=='object'){
		while ((aMatch = re.exec(str)) !== null) {
			matches.add(aMatch[ref]);
			indexes.add(aMatch.index);
		}
	}else{
		while ((aMatch = re.exec(str)) !== null) {
			matches.add(aMatch[ref]);
		}
	}
	return matches;
}

/**
 * Convert lower to upper case
 * @param {string} str - The input string
 * @return {string} The result string
 */
function strtoupper (me, str){
	return str.toUpperCase();
}

/**
 * Convert upper to lower case
 * @param {string} str - The input string
 * @return {string} The result string
 */
function strtolower (me, str){
	return str.toLowerCase();
}

/**
 * Escapes strings for XML
 * @param {string} str - The input string
 * @return {string} The result string
 */
function escapeXml (me, str) {
	return str.replace(/[<>&'"]/g, function (c) {
		switch (c) {
			case '<': return '&lt;';
			case '>': return '&gt;';
			case '&': return '&amp;';
			case "'": return '&apos;';
			case '"': return '&quot;';
		}
	});
}
